;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
;
; File Notes:
; 1. The SPI2 module is configured to communicate with the PIC18F242
;    on the expansion board. The 18F242 will control the LCD on the
;    expansion board.
;
;END_HEADER


	.include "p30f6014.inc"

	.global _init_LCD_Controller	;Provide global scope to routines
	.global	_UpdateLCD
	.global Hello, MsgSwitchPri, MsgSwitchFRC, MsgSwitchLPRC
	.global MsgSwitchDiv1, MsgSwitchDiv4, MsgClockFail, MsgSwitchAbort 

	.section	.data
	.align 2
Hello:			.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               " 
			.ascii "  Clock Switching   " 
			.ascii "  & FSCM            "

MsgSwitchPri:		.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               " 	
			.ascii "Osc Source Switched " 
			.ascii "to Primary Osc      "	
			
MsgSwitchFRC:		.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               " 	
			.ascii "Osc Source Switched " 
			.ascii "to Fast RC Osc      "
			
MsgSwitchLPRC:		.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               " 	
			.ascii "Osc Source Switched " 
			.ascii "to Low-power RC Osc "
			
MsgSwitchDiv1:		.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               "
			.ascii "Postscaler changed  "
			.ascii "to DIV/1            "
			
MsgSwitchDiv4:		.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               "
			.ascii "Postscaler changed  "
			.ascii "to DIV/4            "
			
MsgClockFail:		.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               " 
			.ascii "** CLOCK FAILURE ** " 
			.ascii "Running With Frc Osc"		

MsgSwitchAbort:		.ascii "Masters 2004 806P30 "
			.ascii "Lab3:               " 
			.ascii "**SWITCH ABORTED ** " 
			.ascii "Running on Old Osc  "	
	
	.section 	.text		;Places source code in Code section

_init_LCD_Controller:

	push.d 	w0

;Ensure LCD Controller is up and running	
	mov 	#256, w0
	mov 	#7080, w1
	do 	w1, end_delay1
	repeat 	w0
		nop
		nop
end_delay1:	nop
	
;Reset LCD Controller
	bclr 	SPI2STAT, #SPIROV  		;Clear overflow flag
        bset 	LATG, #LATG9       		;Deselect Slave LCD controller
        bclr 	LATG, #LATG9       		;Select Slave LCD controller

        mov.b 	SPI2BUF, WREG
        mov.b 	#0x80, w0
        mov.b 	WREG, SPI2BUF
poll_port:        
        btss 	SPI2STAT, #SPIRBF
        bra 	poll_port
        mov.b 	SPI2BUF, WREG
        
;Ensure LCD Controller is up and running again
	mov 	#256, w0
	mov 	#7080, w1
	do 	w1, end_delay2
	repeat 	w0
		nop
		nop
end_delay2:	nop
        	
	
;Enable SPI Interrupt Processing
	bclr	IFS1, #SPI2IF
        bset 	IEC1, #SPI2IE      		; Enable SPI2 ISR processing
        
	mov	#Hello, w0
        rcall	_UpdateLCD
	
	pop.d 	w0
	
	return					;Return to calling routine
	
_UpdateLCD:
        push.d 	w0
	
        bclr 	SPI2STAT, #SPIROV  	;Clear overflow flag
        bset 	LATG, #LATG9       	;Deselect Slave LCD controller
        bclr 	LATG, #LATG9       	;Select Slave LCD controller

        clr 	BypassCtrl
        setm 	CtrlOrData

        mov 	#80, w1
        mov 	w1, LcdCharCnt
        
 	mov	w0, LCDDataPtr
                
        mov.b 	SPI2BUF, WREG
        mov.b 	#0x82, w0
        mov.b 	WREG, SPI2BUF	 

        pop.d 	w0
	return				;Return to calling routine	


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.end					;EOF

