;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
;
; File Notes:
; 1. The SPI2 module is configured to communicate with the PIC18F242
;    on the expansion board. The 18F242 will control the LCD on the
;    expansion board.
;
;END_HEADER


	.include "p30f6014.inc"

	.global _LCD_Controller_init	;Provide global scope to routines
	.global	_UpdateLCDString, _DisplayLCD
	.global LCDStringLine1, LCDStringLine2, LCDStringLine3, LCDFreq, LCDPulse
	.global	FreqOrPulseDisplay
	.section	.data
FreqOrPulseDisplay:	.hword 0x0000
	.align 2
LCDStringLine0:		.ascii "Masters 2004: Lab 4 "
LCDStringLine1:		.ascii "Capture & Compare   "
LCDStringLine2:		.ascii "Frequency/Pulse Wdth"
LCDStringLine3:		.ascii "                    "
LCDStringLineF:		.ascii "Frequency= "
LCDFreq:		.ascii "00000 Hz "
LCDStringLineP:		.ascii "Pulse Width="
LCDPulse:		.ascii "00000 us"

	.section 	.text		;Places source code in Code section

_LCD_Controller_init:

	push.d 	w0

;Ensure LCD Controller is up and running	
	mov 	#256, w0
	mov 	#7080, w1
	do 	w1, end_delay1
	repeat 	w0
		nop
		nop
end_delay1:	nop
	
;Reset LCD Controller
	bclr 	SPI2STAT, #SPIROV  		;Clear overflow flag
        bset 	LATG, #LATG9       		;Deselect Slave LCD controller
        bclr 	LATG, #LATG9       		;Select Slave LCD controller

        mov.b 	SPI2BUF, WREG
        mov.b 	#0x80, w0
        mov.b 	WREG, SPI2BUF
poll_port:        
        btss 	SPI2STAT, #SPIRBF
        bra 	poll_port
        mov.b 	SPI2BUF, WREG
        
;Ensure LCD Controller is up and running again
	mov 	#256, w0
	mov 	#7080, w1
	do 	w1, end_delay2
	repeat 	w0
		nop
		nop
end_delay2:	nop
        	
	
;Enable SPI Interrupt Processing
	bclr	IFS1, #SPI2IF
        bset 	IEC1, #SPI2IE      		; Enable SPI2 ISR processing
        
        rcall	_UpdateLCDString
	
	pop.d 	w0
	
	return					;Return to calling routine
	
_UpdateLCDString:
	push RCOUNT
        push.d 	w0

	cp0	FreqOrPulseDisplay
	bra	nz, findPulse
	mov	Frequency, w2
	mov	#0x00E1, w1
	mov	#0x0000, w0
	
	cp0	w2
	bra	z, skip_div1
	repeat	#17
	div.ud	w0, w2
	
	mov w0, Frequency
	clr w1
	rcall	_hextolong
	mov.b	_time10000, WREG
	mov.b	WREG, LCDFreq
	mov.b	_time1000, WREG
	mov.b	WREG, LCDFreq+1
	mov.b	_time100, WREG
	mov.b	WREG, LCDFreq+2
	mov.b	_time10, WREG
	mov.b	WREG, LCDFreq+3
	mov.b	_time1, WREG
	mov.b	WREG, LCDFreq+4
	
	mov	#LCDStringLine3, w1
	mov	#LCDStringLineF, w0
	repeat	#19
	mov.b	[w0++], [w1++]
	
	bra	skipPulse
	
findPulse:
	mov	PulseWidth, w0
	mov	#0x0044, w1
	mul.uu	w0, w1, w0
	
	mov	#0x03E8, w2
	repeat	#17
	div.ud	w0, w2
	
	mov w0, PulseWidth
	clr w1	
	mov	PulseWidth, WREG	
	rcall	_hextolong
	mov.b	_time10000, WREG
	mov.b	WREG, LCDPulse
	mov.b	_time1000, WREG
	mov.b	WREG, LCDPulse+1
	mov.b	_time100, WREG
	mov.b	WREG, LCDPulse+2
	mov.b	_time10, WREG
	mov.b	WREG, LCDPulse+3
	mov.b	_time1, WREG
	mov.b	WREG, LCDPulse+4
	
	mov	#LCDStringLine3, w1
	mov	#LCDStringLineP, w0
	repeat	#19
	mov.b	[w0++], [w1++]
	
	
skipPulse:	
	rcall	_DisplayLCD
	
	
        pop.d 	w0
	pop	RCOUNT
	return				;Return to calling routine	

skip_div1: 
	mov	#0x30, w0
	mov.b	WREG, LCDFreq
	mov.b	WREG, LCDFreq+1
	mov.b	WREG, LCDFreq+2
	mov.b	WREG, LCDFreq+3
	bra	skipPulse


	
_DisplayLCD:
	push	w0
	
	btss	IFS0, #T1IF
	bra	exitdisp
	
        bclr 	SPI2STAT, #SPIROV  	;Clear overflow flag
        bset 	LATG, #LATG9       	;Deselect Slave LCD controller
        bclr 	LATG, #LATG9       	;Select Slave LCD controller

        clr 	BypassCtrl
        setm 	CtrlOrData

        mov 	#80, w0
        mov 	WREG, LcdCharCnt

        mov 	#LCDStringLine0, w0
        mov 	w0, LCDDataPtr
        
        mov.b 	SPI2BUF, WREG
        mov.b 	#0x82, w0
        mov.b 	WREG, SPI2BUF	
        
	bclr IFS0, #T1IF         
exitdisp:
	clr DisplayFlag
        pop	w0
        
        return
         
		


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.end					;EOF

