;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
;
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab4.inc"             ;Lab-specific global equates

        .equ    MAX_PERIOD, 0xFFFF

        .global lab4_part1              ;Provide global scope to routine
        .global __IC6Interrupt          ;Provide global scope to routine

        .section .text                  ;Places source code in Code section


lab4_part1:
        clr   T3CON                     ;Ensure Timer3 module is in a known
                                        ;state
        clr   TMR3                      ;Ensure timer3 counter is reset


        mov   #MAX_PERIOD, w0           ;Load the maximum 16-bit count into
        mov   w0, PR3                   ;PR3

        bclr  IFS0, #T3IF               ;Reset Timer3 interrupt flag

        bclr  IEC0, #T3IE               ;Disable Timer3 ISR processing
                                        ;Timer3 is free-running

        bset    T3CON, #TON             ;Turn on Capture time-base - Timer3

        clr     IC6CON                  ;Clear the IC6CON SFR to a known
                                        ;state

        bset    IC6CON, #ICI1           ;Configure IC6 to interrupt every
        bset    IC6CON, #ICI0           ;4 capture events (using Bit
                                        ;instructions)

        bclr    IC6CON, #ICM2           ;Configure IC6 to capture Timer3
        bset    IC6CON, #ICM1           ;value on every rising edge (using
        bset    IC6CON, #ICM0           ;Bit instructions)

        bclr    IFS1, #IC6IF            ;Clear the IC6 interrupt flag
        bset    IEC1, #IC6IE            ;Enable IC6 ISR processing

        clr     FreqOrPulseDisplay      ;This flag informs LCD display routine
                                        ;to display frequency information when
                                        ;cleared
        return

__IC6Interrupt:

        push.d  w0                      ;Save context of w0 - w5
        push.d  w2
        push.d  w4
        push    RCOUNT                  ;Save RCOUNT context in case a REPEAT
                                        ;loop was interrupted in the foreground
                                        ;process

        clr     w4                      ;Clear  w4, w5
        clr     w5

        mov     IC6BUF, w0              ;Move Input Capture-6 module FIFO
        mov     IC6BUF, w1              ;buffer into w0, w1, w2 and w3
        mov     IC6BUF, w2              ;(in sequence) so that the first
        mov     IC6BUF, w3              ;captured value is in w0 and the last
                                        ;captured value is in w3

        ;Compute the average of four captured values:

        ;First, add the four 16-bit captured values using w5 as a 16-bit
        ; MS Word for storing carry-overs from 16-bit addition
        add     w0, w1, w4              ;Add the first two capture results to w4
        addc    #0, w5                  ;Add the "Carry" bit to w5

        add     w2, w4, w4              ;Add the third captured value to w4
        addc    #0, w5                  ;Add the "Carry" bit to w5

        add     w3, w4, w4              ;Add the fourth captured value to w4
        addc    #0, w5                  ;Add the "Carry" bit to w5


        ;Divide w5:w4 register pair by 4 to compute the average captured value
        ;calculated in the previous 6 instructions

        mov     #4, w2                  ;Load w2(divisor) with 4
        repeat  #17                     ;Set up REPEAT loop for division
        div.ud  w4, w2                  ;Perform Unsigned 32-bit/16-bit division
                                        ;so that w0 = quotient of (w5:w4)/w2

        mov     w0, Frequency           ;Store quotient to the variable
                                        ;"Frequency"
        setm    DisplayFlag             ;Set the DisplayFlag variable

        pop     RCOUNT                  ;Restore context of RCOUNT
        pop.d   w4                      ;Restore context of w0 - w5
        pop.d   w2
        pop.d   w0
        bclr    IFS1, #IC6IF            ;Clear the IC6 interrupt flag
        retfie                          ;Return from Interrupt

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.end                                            ;EOF



